{% extends 'django_spire/page/full_page.html' %}

{% block full_page_content %}
    {% block infinite_scroll_container %}
        {% include 'django_spire/container/infinite_scroll_container.html' %}
    {% endblock %}
{% endblock %}
