{% extends 'django_spire/dropdown/ellipsis_dropdown.html' %}

{% block dropdown_content %}
    {% if AuthController.knowledge.can_change %}
        {% include 'django_spire/dropdown/element/dropdown_link_element.html' with x_link_url='entry.edit_url' link_icon='bi bi-input-cursor-text' link_text='Rename' %}
        {% include 'django_spire/dropdown/element/dropdown_link_element.html' with x_link_url='entry.edit_version_url' link_icon='bi bi-pencil' link_text='Edit' %}
    {% endif %}
    {% if AuthController.knowledge.can_delete %}
        {% include 'django_spire/dropdown/element/dropdown_link_element.html' with x_link_url='entry.delete_url' link_icon='bi bi-trash3' link_text='Delete' link_css='text-app-danger' %}
    {% endif %}
{% endblock %}

