# Generated by Django 5.2.1 on 2025-05-28 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_notification_email', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='bcc',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='cc',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='context_data',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='template_id',
            field=models.CharField(default='', max_length=64),
        ),
    ]
