{% extends 'django_spire/filtering/form/base_session_filter_form.html' %}

{% block filter_type %}search{% endblock %}

{% block submit_button %}
    {% include 'django_spire/contrib/form/button/form_submit_button.html' with button_text='Search' %}
{% endblock %}
