{% extends 'django_spire/card/card.html' %}

{% block card_padding %}
	px-3 py-2
{% endblock %}

{% block card_content %}
    <div class="row">
        <div class="col-12 vh-minus-top-nav">
            {% include 'django_spire/ai/chat/widget/selection_widget.html' %}
        </div>
    </div>
{% endblock %}
