<a
    class="btn shadow-sm {% block button_class %}{% endblock %} {{ button_class|default:'btn-sm' }}"
    {% block button_attributes %}{% endblock %}
    {% if button_href %}href="{{ button_href }}{{ button_url_params }}"{% endif %}
    {% if x_button_click %}@click="{{ x_button_click }}"{% endif %}
>
    {% if button_icon %}
        <i class="{{ button_icon }}"></i>
    {% endif %}

    {{ button_text }}

    {% if x_button_text %}
        <span x-text="{{ x_button_text }}"></span>
    {% endif %}
</a>
