[data-theme="dark"] {
    --app-primary: #81a1c1;
    --app-primary-soft: #1f3140;
    --app-primary-dark: #5e81ac;

    --app-secondary: #d8dee9;
    --app-secondary-soft: #424952;
    --app-secondary-dark: #e5e9f0;

    --app-accent: #bf616a;
    --app-accent-soft: #3d2a2d;
    --app-accent-dark: #a54e56;

    --app-success: #a3be8c;
    --app-success-soft: #232e24;
    --app-success-dark: #8fa876;

    --app-warning: #ebcb8b;
    --app-warning-soft: #3d3527;
    --app-warning-dark: #d4b066;

    --app-danger: #bf616a;
    --app-danger-soft: #3d2a2d;
    --app-danger-dark: #a54e56;

    --app-layer-one: #2e3440;
    --app-layer-two: #3b4252;
    --app-layer-three: #434c5e;
    --app-layer-four: #4c566a;

    --app-alt-layer-one: #242933;
    --app-alt-layer-two: #353b48;
    --app-alt-layer-three: #3e4758;
    --app-alt-layer-four: #475164;

    --app-card-header: #3b4252;

    --app-default-attribute-title-color: #eceff4;
    --app-default-button-text-color: #2e3440;
    --app-default-text-color: #eceff4;
    --app-default-alt-text-color: #d8dee9;
    --app-default-link-color: #81a1c1;
    --app-default-link-hover-color: #bf616a;

    --app-inverted-text-color: #2e3440;
    --app-inverted-alt-text-color: #3b4252;

    --app-side-navigation-bg-color: #2e3440;
    --app-side-navigation-text-color: #eceff4;
    --app-side-navigation-link-color: #eceff4;
    --app-side-navigation-link-hover-color: #88c0d0;

    --app-top-navigation-bg-color: #2e3440;
    --app-top-navigation-text-color: #eceff4;
    --app-top-navigation-link-color: #81a1c1;
    --app-top-navigation-link-hover-color: #bf616a;

    --app-footer-bg-color: #2e3440;
    --app-footer-text-color: #eceff4;
    --app-footer-link-color: #81a1c1;
    --app-footer-link-hover-color: #bf616a;

    --app-table-row-color: #2e3440;
    --app-table-row-alt-color: #3b4252;
    --app-table-row-hover-color: #434c5e;

    --bs-border-color: #4c566a;

    --app-shadow-light: rgba(0, 0, 0, 0.3);
    --app-shadow-medium: rgba(0, 0, 0, 0.5);
    --app-shadow-heavy: rgba(0, 0, 0, 0.7);
    --app-shadow-navigation: rgba(0, 0, 0, 0.9);

    --app-border-translucent-light: rgba(76, 86, 106, 0.3);
    --app-focus-ring-primary: rgba(129, 161, 193, 0.25);
}
