<div class="row {% block container_outer_class %}{% endblock %}">
    <div class="col-12">
        <div class="my-4 pb-4 px-md-3">
            <div class="align-items-center d-flex justify-content-between mx-auto" style="max-width: 1200px;">
                <div class="flex-grow-1 text-center">
                    <h1 class="fs-2 fw-bold mb-2 text-app-primary">
                        {% block container_title %}
                        {% endblock %}
                    </h1>
                    <div class="fs-6 text-alt-color">
                        {% block container_subtitle %}
                        {% endblock %}
                    </div>
                </div>

                <div class="flex-shrink-0 ms-3">
                    {% block container_button %}
                    {% endblock %}
                </div>
            </div>
        </div>

        {% block container_filter_section %}
        {% endblock %}

        <div class="row {% block container_class %}{% endblock %}">
            <div class="col-12 {% block container_content_class %}{% endblock %}">
                {% block container_content %}
                {% endblock %}
            </div>
        </div>
    </div>
</div>
