<div class="row align-items-center g-2">
    <div class="col-auto">
        <div class="mb-0 fs--1 text-app-secondary fw-bold">
            {{ divider_title }}
        </div>
    </div>

    <div class="col">
        <hr class="m-0 text-app-secondary" style="height: 1x !important;">
    </div>
</div>
