{% extends "django_spire/notification/app/dropdown/notification_dropdown.html" %}

{% block notification_dropdown_trigger %}
    <div class="dropdown">
        <i x-bind='trigger' class="bi bi-bell-fill fs-2 cursor-pointer mx-2"></i>
        <span x-show="new_notification" x-cloak x-transition style="width: 10px; height: 10px; top: 8px; right: 10px; position: absolute;" class="bg-danger rounded-circle"></span>
    </div>
{% endblock %}
